(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue'), require('vue-class-component'), require('vuex')) :
	typeof define === 'function' && define.amd ? define(['exports', 'vue', 'vue-class-component', 'vuex'], factory) :
	(factory((global.VueTypescript = {}),global.Vue,global.VueClassComponent,global.Vuex));
}(this, (function (exports,vue,vueClassComponent,vuex) { 'use strict';

vue = vue && vue.hasOwnProperty('default') ? vue['default'] : vue;
var vueClassComponent__default = 'default' in vueClassComponent ? vueClassComponent['default'] : vueClassComponent;

function createExDecorator(factory) {
    return function (target, key, descriptor) {
        var c = vueClassComponent.createDecorator(function (o, k) {
            return factory(o, k, descriptor);
        });
        return c(target, key);
    };
}

function Filter() {
    return createExDecorator(function (componentOptions, k, descriptor) {
        if (typeof componentOptions.filters !== 'object') {
            componentOptions.filters = Object.create(null);
        }
        componentOptions.filters[k] = descriptor.value;
    });
}

function Prop(options) {
    if (options === void 0) { options = {}; }
    return vueClassComponent.createDecorator(function (componentOptions, k) {
        if (typeof componentOptions.props !== 'object') {
            componentOptions.props = Object.create(null);
        }
        componentOptions.props[k] = options;
    });
}

function Watch(path, options) {
    if (options === void 0) { options = {}; }
    var _a = options.deep, deep = _a === void 0 ? false : _a, _b = options.immediate, immediate = _b === void 0 ? false : _b;
    return vueClassComponent.createDecorator(function (componentOptions, handler) {
        if (typeof componentOptions.watch !== 'object') {
            componentOptions.watch = Object.create(null);
        }
        componentOptions.watch[path] = { handler: handler, deep: deep, immediate: immediate };
    });
}

function isVuexModuleKey(key) {
    return ['namespaced', 'state', 'getters', 'actions', 'mutations', 'modules'].indexOf(key) >= 0;
}
function shouldAddPropertyToState(state, key) {
    return state.hasOwnProperty(key) &&
        typeof state[key] !== 'function' &&
        !isVuexModuleKey(key);
}
function moduleDecoratorFactory(modOrOpt) {
    return function (constructor) {
        var module = constructor;
        var state = new (module.prototype.constructor)({});
        if (typeof module.state !== 'object') {
            module.state = Object.create(null);
        }
        module.namespaced = modOrOpt && modOrOpt.namespaced;
        Object.keys(state).forEach(function (key) {
            if (shouldAddPropertyToState(state, key)) {
                module.state[key] = state[key];
            }
        });
    };
}
var VuexModule = /** @class */ (function () {
    function VuexModule(module) {
        this.actions = module.actions;
        this.mutations = module.mutations;
        this.state = module.state;
        this.getters = module.getters;
        this.namespaced = module.namespaced;
        this.modules = module.modules;
    }
    return VuexModule;
}());
function Module(modOrOpt) {
    if (typeof modOrOpt === 'function') {
        return moduleDecoratorFactory({})(modOrOpt);
    }
    else {
        return moduleDecoratorFactory(modOrOpt);
    }
}

function createVuexDecorator(factory) {
    return function (target, key, descriptor) {
        var options = typeof target === 'function'
            ? target
            : target.constructor;
        factory(options, key, descriptor);
    };
}

function checkOptionsCommit(options, context, value) {
    if (options && options.commit) {
        context.commit(options.commit, value);
    }
}
function createActionFunction(key, value, options) {
    var action = function (context, payload) {
        try {
            var callResult_1 = value.call(context, payload);
            if (Promise.resolve(callResult_1) !== callResult_1) {
                checkOptionsCommit(options, context, callResult_1);
                return callResult_1;
            }
            return new Promise(function (resolve, reject) {
                callResult_1.then(function (result) {
                    checkOptionsCommit(options, context, result);
                    resolve(result);
                }, function (error) {
                    reject(error);
                });
            });
        }
        catch (ex) {
            console.error('Could not perform action ' + key.toString());
        }
    };
    return action;
}
function Action(options) {
    if (options === void 0) { options = {}; }
    return createVuexDecorator(function (componentOptions, k, description) {
        if (typeof componentOptions.actions !== 'object') {
            componentOptions.actions = Object.create(null);
        }
        componentOptions.actions[k] = createActionFunction(k, description.value, options);
    });
}

function createMutationFunction(value) {
    var mutation = function (state, payload) {
        value.call(state, payload);
    };
    return mutation;
}
function Mutation() {
    return createVuexDecorator(function (componentOptions, k, description) {
        if (typeof componentOptions.mutations !== 'object') {
            componentOptions.mutations = Object.create(null);
        }
        componentOptions.mutations[k] = createMutationFunction(description.value);
    });
}

function createGetterFunction(value) {
    var getter = function (state) {
        return value.call(state);
    };
    return getter;
}
function Getter() {
    return createVuexDecorator(function (componentOptions, k, description) {
        if (typeof componentOptions.getters !== 'object') {
            componentOptions.getters = Object.create(null);
        }
        componentOptions.getters[k] = createGetterFunction(description.value);
    });
}

function MapAction(options) {
    if (options === void 0) { options = {}; }
    var _a = options.namespace, namespace = _a === void 0 ? undefined : _a;
    return vueClassComponent.createDecorator(function (componentOptions, k) {
        if (typeof componentOptions.methods !== 'object') {
            componentOptions.methods = Object.create(null);
        }
        var mapArray = [k];
        componentOptions.methods[k] = namespace !== undefined
            ? vuex.mapActions(namespace, mapArray)[k]
            : vuex.mapActions(mapArray)[k];
    });
}

function MapGetter(options) {
    if (options === void 0) { options = {}; }
    var _a = options.namespace, namespace = _a === void 0 ? undefined : _a;
    return vueClassComponent.createDecorator(function (componentOptions, k) {
        if (typeof componentOptions.computed !== 'object') {
            componentOptions.computed = Object.create(null);
        }
        var mapArray = [k];
        componentOptions.computed[k] = namespace !== undefined
            ? vuex.mapGetters(namespace, mapArray)[k]
            : vuex.mapGetters(mapArray)[k];
    });
}

exports.Vue = vue;
exports.Component = vueClassComponent__default;
exports.createDecorator = vueClassComponent.createDecorator;
exports.mixins = vueClassComponent.mixins;
exports.Filter = Filter;
exports.Prop = Prop;
exports.Watch = Watch;
exports.VuexModule = VuexModule;
exports.Module = Module;
exports.Action = Action;
exports.Mutation = Mutation;
exports.Getter = Getter;
exports.MapAction = MapAction;
exports.MapGetter = MapGetter;

Object.defineProperty(exports, '__esModule', { value: true });

})));
